@@ -57,21 +57,12 @@
 #define KEY_TERMINAL_DIR "/desktop/gnome/applications/terminal"
 #define KEY_TERMINAL_COMMAND KEY_TERMINAL_DIR "/exec"
 
-#define KEY_OVERLAY_KEY "/apps/metacity/general/overlay_key"
 #define KEY_SCREEN_BINDINGS_PREFIX "/apps/metacity/global_keybindings"
 #define KEY_WINDOW_BINDINGS_PREFIX "/apps/metacity/window_keybindings"
 #define KEY_LIST_BINDINGS_SUFFIX "_list"
 
 #define KEY_WORKSPACE_NAME_PREFIX "/apps/metacity/workspace_names/name_"
 
-#ifdef WITH_CLUTTER
-#define KEY_CLUTTER_DISABLED "/apps/metacity/general/clutter_disabled"
-#define KEY_CLUTTER_PLUGINS  "/apps/metacity/general/clutter_plugins"
-#endif
-
-#define KEY_LIVE_HIDDEN_WINDOWS "/apps/metacity/general/live_hidden_windows"
-
-#define KEY_NO_TAB_POPUP "/apps/metacity/general/no_tab_popup"
 
 #ifdef HAVE_GCONF
 static GConfClient *default_client = NULL;
@@ -104,6 +95,7 @@ static char *cursor_theme = NULL;
 static int   cursor_size = 24;
 static gboolean compositing_manager = FALSE;
 static gboolean resize_with_right_button = FALSE;
+static gboolean force_fullscreen = TRUE;
 
 static MetaVisualBellType visual_bell_type = META_VISUAL_BELL_FULLSCREEN_FLASH;
 static MetaButtonLayout button_layout;
@@ -115,16 +107,6 @@ static char *terminal_command = NULL;
 
 static char *workspace_names[MAX_REASONABLE_WORKSPACES] = { NULL, };
 
-#ifdef WITH_CLUTTER
-static gboolean clutter_disabled = FALSE;
-static gboolean clutter_plugins_overridden = FALSE;
-static GSList *clutter_plugins = NULL;
-#endif
-
-static gboolean live_hidden_windows = FALSE;
-
-static gboolean no_tab_popup = FALSE;
-
 #ifdef HAVE_GCONF
 static gboolean handle_preference_update_enum (const gchar *key, GConfValue *value);
 
@@ -428,23 +410,6 @@ static MetaBoolPreference preferences_bool[] =
       &resize_with_right_button,
       FALSE,
     },
-#ifdef WITH_CLUTTER
-    { "/apps/metacity/general/clutter_disabled",
-      META_PREF_CLUTTER_DISABLED,
-      &clutter_disabled,
-      FALSE,
-    },
-#endif
-    { "/apps/metacity/general/live_hidden_windows",
-      META_PREF_LIVE_HIDDEN_WINDOWS,
-      &live_hidden_windows,
-      FALSE,
-    },
-    { "/apps/metacity/general/no_tab_popup",
-      META_PREF_NO_TAB_POPUP,
-      &no_tab_popup,
-      FALSE,
-    },
     { NULL, 0, NULL, FALSE },
   };
 
@@ -578,13 +543,7 @@ handle_preference_init_string (void)
       value = gconf_client_get_string (default_client,
                                        cursor->key,
                                        &error);
-
-      if (error || !value)
-        {
-          cleanup_error (&error);
-          ++cursor;
-          continue;
-        }
+      cleanup_error (&error);
 
       if (cursor->handler)
         {
@@ -1067,14 +1026,6 @@ meta_prefs_init (void)
   handle_preference_init_string ();
   handle_preference_init_int ();
 
-#ifdef WITH_CLUTTER
-  if (!clutter_plugins_overridden)
-    clutter_plugins = gconf_client_get_list (default_client, KEY_CLUTTER_PLUGINS,
-                                             GCONF_VALUE_STRING, &err);
-
-  cleanup_error (&err);
-#endif
-
   /* @@@ Is there any reason we don't do the add_dir here? */
   for (gconf_dir_cursor=gconf_dirs_we_are_interested_in;
        *gconf_dir_cursor!=NULL;
@@ -1218,29 +1169,6 @@ change_notify (GConfClient    *client,
       if (update_workspace_name (key, str))
         queue_changed (META_PREF_WORKSPACE_NAMES);
     }
-  else if (g_str_equal (key, KEY_OVERLAY_KEY))
-    {
-      queue_changed (META_PREF_KEYBINDINGS);
-    }
-#ifdef WITH_CLUTTER
-  else if (g_str_equal (key, KEY_CLUTTER_PLUGINS) && !clutter_plugins_overridden)
-    {
-      GError *err = NULL;
-      GSList *l;
-
-      l = gconf_client_get_list (default_client, KEY_CLUTTER_PLUGINS,
-                                 GCONF_VALUE_STRING, &err);
-
-      if (!l)
-        {
-          cleanup_error (&err);
-          goto out;
-        }
-
-      clutter_plugins = l;
-      queue_changed (META_PREF_CLUTTER_PLUGINS);
-    }
-#endif
   else
     {
       meta_topic (META_DEBUG_PREFS, "Key %s doesn't mean anything to Metacity\n",
@@ -1824,16 +1752,9 @@ meta_preference_to_string (MetaPreference pref)
 
     case META_PREF_RESIZE_WITH_RIGHT_BUTTON:
       return "RESIZE_WITH_RIGHT_BUTTON";
-#ifdef WITH_CLUTTER
-    case META_PREF_CLUTTER_DISABLED:
-      return "CLUTTER_DISABLED";
-    case META_PREF_CLUTTER_PLUGINS:
-      return "CLUTTER_PLUGINS";
-#endif
-    case META_PREF_LIVE_HIDDEN_WINDOWS:
-      return "LIVE_HIDDEN_WINDOWS";
-    case META_PREF_NO_TAB_POPUP:
-      return "NO_TAB_POPUP";
+
+    case META_PREF_FORCE_FULLSCREEN:
+      return "FORCE_FULLSCREEN";
     }
 
   return "(unknown)";
@@ -1871,45 +1792,45 @@ meta_prefs_set_num_workspaces (int n_workspaces)
 }
 
 #define keybind(name, handler, param, flags, stroke, description) \
-  { #name, stroke, NULL, !!(flags & BINDING_REVERSES), !!(flags & BINDING_PER_WINDOW) },
+  { #name, NULL, !!(flags & BINDING_REVERSES), !!(flags & BINDING_PER_WINDOW) },
 static MetaKeyPref key_bindings[] = {
 #include "all-keybindings.h"
-  { NULL, NULL, NULL, FALSE }
+  { NULL, NULL, FALSE }
 };
 #undef keybind
 
-static MetaKeyCombo overlay_key_combo = { 0, 0, 0 };
+#ifndef HAVE_GCONF
 
-/* These bindings are for modifiers alone, so they need special handling */
-static void
-init_special_bindings (void)
+/**
+ * A type to map names of keybindings (such as "switch_windows")
+ * to the binding strings themselves (such as "<Alt>Tab").
+ * It exists only when GConf is turned off in ./configure and
+ * functions as a sort of ersatz GConf.
+ */
+typedef struct
 {
-#ifdef HAVE_GCONF
-  char *val;
-  GError *err = NULL;
-#endif
-  
-  /* Default values for bindings which are global, but take special handling */
-  meta_ui_parse_accelerator ("Super_L", &overlay_key_combo.keysym, 
-                             &overlay_key_combo.keycode, 
-                             &overlay_key_combo.modifiers);
+  const char *name;
+  const char *keybinding;
+} MetaSimpleKeyMapping;
 
-#ifdef HAVE_GCONF
-  val = gconf_client_get_string (default_client, KEY_OVERLAY_KEY, &err);
-  cleanup_error (&err);
-    
-  if (val && meta_ui_parse_accelerator (val, &overlay_key_combo.keysym, 
-                                        &overlay_key_combo.keycode, 
-                                        &overlay_key_combo.modifiers))
-    ;
-  else
-    {
-      meta_topic (META_DEBUG_KEYBINDINGS,
-                  "Failed to parse value for overlay_key\n");
-    }
-  g_free (val);
-#endif
-}
+/* FIXME: This would be neater if the array only contained entries whose
+ * default keystroke was non-null.  You COULD do this by defining
+ * ONLY_BOUND_BY_DEFAULT around various blocks at the cost of making
+ * the bindings file way more complicated.  However, we could stop this being
+ * data and move it into code.  Then the compiler would optimise away
+ * the problem lines.
+ */
+
+#define keybind(name, handler, param, flags, stroke, description) \
+  { #name, stroke },
+
+static MetaSimpleKeyMapping key_string_bindings[] = {
+#include "all-keybindings.h"
+  { NULL, NULL }
+};
+#undef keybind
+
+#endif /* NOT HAVE_GCONF */
 
 static void
 init_bindings (void)
@@ -1964,22 +1885,27 @@ init_bindings (void)
 
       ++i;
     }
-
 #else /* HAVE_GCONF */
   int i = 0;
-  while (key_bindings[i].name)
+  int which = 0;
+  while (key_string_bindings[i].name)
     {
-      if (key_bindings[i].default_keybinding)
-        {
-          /* Set the binding */
-          update_binding (&key_bindings[i], NULL);
-        }
+      if (key_string_bindings[i].keybinding == NULL) {
+        ++i;
+        continue;
+      }
+    
+      while (strcmp(key_bindings[which].name, 
+                    key_string_bindings[i].name) != 0)
+        which++;
+
+      /* Set the binding */
+      update_binding (&key_bindings[which], 
+                      key_string_bindings[i].keybinding);
 
       ++i;
     }
 #endif /* HAVE_GCONF */
-  
-  init_special_bindings ();  
 }
 
 static void
@@ -2063,9 +1989,6 @@ update_binding (MetaKeyPref *binding,
   MetaKeyCombo *combo;
   gboolean changed;
 
-  if (value == NULL)
-    value = binding->default_keybinding;
-
   meta_topic (META_DEBUG_KEYBINDINGS,
               "Binding \"%s\" has new gconf value \"%s\"\n",
               binding->name, value ? value : "none");
@@ -2691,12 +2614,6 @@ meta_prefs_get_key_bindings (const MetaKeyPref **bindings,
   *n_bindings = (int) G_N_ELEMENTS (key_bindings) - 1;
 }
 
-void 
-meta_prefs_get_overlay_binding (MetaKeyCombo *combo)
-{
-  *combo = overlay_key_combo;
-}
-
 MetaActionTitlebar
 meta_prefs_get_action_double_click_titlebar (void)
 {
@@ -2824,6 +2741,12 @@ meta_prefs_get_mouse_button_menu (void)
   return resize_with_right_button ? 2: 3;
 }
 
+gboolean
+meta_prefs_get_force_fullscreen (void)
+{
+  return force_fullscreen;
+}
+
 void
 meta_prefs_set_compositing_manager (gboolean whether)
 {
@@ -2846,141 +2769,6 @@ meta_prefs_set_compositing_manager (gboolean whether)
 #endif
 }
 
-#ifdef WITH_CLUTTER
-gboolean
-meta_prefs_get_clutter_disabled (void)
-{
-  return clutter_disabled;
-}
-
-void
-meta_prefs_set_clutter_disabled (gboolean whether)
-{
-#ifdef HAVE_GCONF
-  GError *err = NULL;
-
-  gconf_client_set_bool (default_client,
-                         KEY_CLUTTER_DISABLED,
-                         whether,
-                         &err);
-
-  if (err)
-    {
-      meta_warning (_("Error setting clutter status status: %s\n"),
-                    err->message);
-      g_error_free (err);
-    }
-#else
-  clutter_disabled = whether;
-#endif
-}
-
-GSList *
-meta_prefs_get_clutter_plugins (void)
-{
-  return clutter_plugins;
-}
-
-void
-meta_prefs_set_clutter_plugins (GSList *list)
-{
-  GError *err = NULL;
-
-  gconf_client_set_list (default_client,
-                         KEY_CLUTTER_PLUGINS,
-                         GCONF_VALUE_STRING,
-                         list,
-                         &err);
-
-  if (err)
-    {
-      meta_warning (_("Error setting clutter plugin list: %s\n"),
-                    err->message);
-      g_error_free (err);
-    }
-}
-
-void
-meta_prefs_override_clutter_plugins (GSList *list)
-{
-  GSList *l;
-
-  clutter_plugins_overridden = TRUE;
-  clutter_plugins = NULL;
-
-  for (l = list; l; l = l->next)
-    clutter_plugins = g_slist_prepend (clutter_plugins, g_strdup(l->data));
-
-  clutter_plugins = g_slist_reverse (clutter_plugins);
-}
-#endif
-
-gboolean
-meta_prefs_get_live_hidden_windows (void)
-{
-#if 0
-  return live_hidden_windows;
-#else
-  return TRUE;
-#endif
-}
-
-void
-meta_prefs_set_live_hidden_windows (gboolean whether)
-{
-#ifdef HAVE_GCONF
-  GError *err = NULL;
-
-  gconf_client_set_bool (default_client,
-                         KEY_LIVE_HIDDEN_WINDOWS,
-                         whether,
-                         &err);
-
-  if (err)
-    {
-      meta_warning (_("Error setting live hidden windows status status: %s\n"),
-                    err->message);
-      g_error_free (err);
-    }
-#else
-  live_hidden_windows = whether;
-#endif
-}
-
-gboolean
-meta_prefs_get_no_tab_popup (void)
-{
-  return no_tab_popup;
-}
-
-void
-meta_prefs_set_no_tab_popup (gboolean whether)
-{
-#ifdef HAVE_GCONF
-  GError *err = NULL;
-
-  gconf_client_set_bool (default_client,
-                         KEY_NO_TAB_POPUP,
-                         whether,
-                         &err);
-
-  if (err)
-    {
-      meta_warning (_("Error setting no tab popup status: %s\n"),
-                    err->message);
-      g_error_free (err);
-    }
-#else
-  no_tab_popup = whether;
-#endif
-}
-
-void
-meta_prefs_override_no_tab_popup (gboolean whether)
-{
-  no_tab_popup = whether;
-}
-
 #ifndef HAVE_GCONF
 static void
 init_button_layout(void)
@@ -3019,3 +2807,10 @@ init_button_layout(void)
 };
 
 #endif
+
+void
+meta_prefs_set_force_fullscreen (gboolean whether)
+{
+  force_fullscreen = whether;
+}
+
