@@ -25,22 +25,11 @@
 
 #include <config.h>
 #include "workspace.h"
-#include "workspace-private.h"
 #include "errors.h"
 #include "prefs.h"
-
-#ifdef HAVE_COMPOSITE_EXTENSIONS
-#include "compositor.h"
-#endif
-
 #include <X11/Xatom.h>
 #include <string.h>
-
-enum {
-  PROP_0,
-
-  PROP_N_WINDOWS,
-};
+#include <canberra-gtk.h>
 
 void meta_workspace_queue_calc_showing   (MetaWorkspace *workspace);
 static void set_active_space_hint        (MetaScreen *screen);
@@ -49,104 +38,7 @@ static void focus_ancestor_or_mru_window (MetaWorkspace *workspace,
                                           guint32        timestamp);
 static void free_this                    (gpointer candidate,
                                           gpointer dummy);
-
-G_DEFINE_TYPE (MetaWorkspace, meta_workspace, G_TYPE_OBJECT);
-
-enum
-{
-  WINDOW_ADDED,
-  WINDOW_REMOVED,
-
-  LAST_SIGNAL
-};
-
-static guint signals [LAST_SIGNAL] = { 0 };
-
-static void
-meta_workspace_finalize (GObject *object)
-{
-  /* Actual freeing done in meta_workspace_remove() for now */
-}
-
-static void
-meta_workspace_set_property (GObject      *object,
-                             guint         prop_id,
-                             const GValue *value,
-                             GParamSpec   *pspec)
-{
-  switch (prop_id)
-    {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
-static void
-meta_workspace_get_property (GObject      *object,
-                             guint         prop_id,
-                             GValue       *value,
-                             GParamSpec   *pspec)
-{
-  MetaWorkspace *ws = META_WORKSPACE (object);
-
-  switch (prop_id)
-    {
-    case PROP_N_WINDOWS:
-      /*
-       * This is reliable, but not very efficient; should we store
-       * the list lenth ?
-       */
-      g_value_set_uint (value, g_list_length (ws->windows));
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-    }
-}
-
-static void
-meta_workspace_class_init (MetaWorkspaceClass *klass)
-{
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  GParamSpec   *pspec;
-
-  object_class->finalize     = meta_workspace_finalize;
-  object_class->get_property = meta_workspace_get_property;
-  object_class->set_property = meta_workspace_set_property;
-
-  signals[WINDOW_ADDED] = g_signal_new ("window-added",
-                                        G_TYPE_FROM_CLASS (klass),
-                                        G_SIGNAL_RUN_LAST,
-                                        0,
-                                        NULL, NULL,
-                                        g_cclosure_marshal_VOID__OBJECT,
-                                        G_TYPE_NONE, 1,
-                                        META_TYPE_WINDOW);
-  signals[WINDOW_REMOVED] = g_signal_new ("window-removed",
-                                          G_TYPE_FROM_CLASS (klass),
-                                          G_SIGNAL_RUN_LAST,
-                                          0,
-                                          NULL, NULL,
-                                          g_cclosure_marshal_VOID__OBJECT,
-                                          G_TYPE_NONE, 1,
-                                          META_TYPE_WINDOW);
-
-  pspec = g_param_spec_uint ("n-windows",
-                             "N Windows",
-                             "Number of windows",
-                             0, G_MAXUINT, 0,
-                             G_PARAM_READABLE);
-
-  g_object_class_install_property (object_class,
-                                   PROP_N_WINDOWS,
-                                   pspec);
-}
-
-static void
-meta_workspace_init (MetaWorkspace *workspace)
-{
-}
+static void workspace_free_struts        (MetaWorkspace *workspace);
 
 static void
 maybe_add_to_list (MetaScreen *screen, MetaWindow *window, gpointer data)
@@ -162,7 +54,7 @@ meta_workspace_new (MetaScreen *screen)
 {
   MetaWorkspace *workspace;
 
-  workspace = g_object_new (META_TYPE_WORKSPACE, NULL);
+  workspace = g_new (MetaWorkspace, 1);
 
   workspace->screen = screen;
   workspace->screen->workspaces =
@@ -184,7 +76,6 @@ meta_workspace_new (MetaScreen *screen)
   workspace->xinerama_edges = NULL;
   workspace->list_containing_self = g_list_prepend (NULL, workspace);
 
-  workspace->builtin_struts = NULL;
   workspace->all_struts = NULL;
 
   workspace->showing_desktop = FALSE;
@@ -200,12 +91,12 @@ free_this (gpointer candidate, gpointer dummy)
 }
 
 /**
- * Frees the combined struts list of a workspace.
+ * Frees the struts list of a workspace.
  *
  * \param workspace  The workspace.
  */
 static void
-workspace_free_all_struts (MetaWorkspace *workspace)
+workspace_free_struts (MetaWorkspace *workspace)
 {
   if (workspace->all_struts == NULL)
     return;
@@ -215,24 +106,8 @@ workspace_free_all_struts (MetaWorkspace *workspace)
   workspace->all_struts = NULL;
 }
 
-/**
- * Frees the struts list set with meta_workspace_set_builtin_struts
- *
- * \param workspace  The workspace.
- */
-static void
-workspace_free_builtin_struts (MetaWorkspace *workspace)
-{
-  if (workspace->builtin_struts == NULL)
-    return;
-    
-  g_slist_foreach (workspace->builtin_struts, free_this, NULL);
-  g_slist_free (workspace->builtin_struts);
-  workspace->builtin_struts = NULL;
-}
-
 void
-meta_workspace_remove (MetaWorkspace *workspace)
+meta_workspace_free (MetaWorkspace *workspace)
 {
   GList *tmp;
   MetaScreen *screen;
@@ -270,8 +145,6 @@ meta_workspace_remove (MetaWorkspace *workspace)
   g_list_free (workspace->mru_list);
   g_list_free (workspace->list_containing_self);
 
-  workspace_free_builtin_struts (workspace);
-
   /* screen.c:update_num_workspaces(), which calls us, removes windows from
    * workspaces first, which can cause the workareas on the workspace to be
    * invalidated (and hence for struts/regions/edges to be freed).
@@ -281,7 +154,7 @@ meta_workspace_remove (MetaWorkspace *workspace)
 
   if (!workspace->work_areas_invalid)
     {
-      workspace_free_all_struts (workspace);
+      workspace_free_struts (workspace);
       for (i = 0; i < screen->n_xinerama_infos; i++)
         meta_rectangle_free_list_and_elements (workspace->xinerama_region[i]);
       g_free (workspace->xinerama_region);
@@ -290,7 +163,7 @@ meta_workspace_remove (MetaWorkspace *workspace)
       meta_rectangle_free_list_and_elements (workspace->xinerama_edges);
     }
 
-  g_object_unref (workspace);
+  g_free (workspace);
 
   /* don't bother to reset names, pagers can just ignore
    * extra ones
@@ -328,7 +201,6 @@ meta_workspace_add_window (MetaWorkspace *workspace,
     }
 
   workspace->windows = g_list_prepend (workspace->windows, window);
-
   window->workspace = workspace;
 
   meta_window_set_current_workspace_hint (window);
@@ -345,9 +217,6 @@ meta_workspace_add_window (MetaWorkspace *workspace,
    * the relevant struts
    */
   meta_window_queue (window, META_QUEUE_CALC_SHOWING|META_QUEUE_MOVE_RESIZE);
-
-  g_signal_emit (workspace, signals[WINDOW_ADDED], 0, window);
-  g_object_notify (G_OBJECT (workspace), "n-windows");
 }
 
 void
@@ -394,9 +263,6 @@ meta_workspace_remove_window (MetaWorkspace *workspace,
    * the relevant struts
    */
   meta_window_queue (window, META_QUEUE_CALC_SHOWING|META_QUEUE_MOVE_RESIZE);
-
-  g_signal_emit (workspace, signals[WINDOW_REMOVED], 0, window);
-  g_object_notify (G_OBJECT (workspace), "n-windows");
 }
 
 void
@@ -435,24 +301,69 @@ meta_workspace_queue_calc_showing  (MetaWorkspace *workspace)
   tmp = workspace->windows;
   while (tmp != NULL)
     {
-      if (meta_prefs_get_live_hidden_windows ())
-        {
-          /*
-           * When we hide rather than unmap windows, we need the show/hide
-           * status of the window to be recalculated *before* we call the
-           * compositor switch_workspace hook.
-           */
-          meta_window_calc_showing (tmp->data);
-        }
-      else
-        {
-          meta_window_queue (tmp->data, META_QUEUE_CALC_SHOWING);
-        }
+      meta_window_queue (tmp->data, META_QUEUE_CALC_SHOWING);
 
       tmp = tmp->next;
     }
 }
 
+static void workspace_switch_sound(MetaWorkspace *from,
+                                   MetaWorkspace *to) {
+
+  MetaWorkspaceLayout layout;
+  int i, nw, x, y, fi, ti;
+  const char *e;
+
+  nw = meta_screen_get_n_workspaces(from->screen);
+  fi = meta_workspace_index(from);
+  ti = meta_workspace_index(to);
+
+  meta_screen_calc_workspace_layout(from->screen,
+                                    nw,
+                                    fi,
+                                    &layout);
+
+  for (i = 0; i < nw; i++)
+    if (layout.grid[i] == ti)
+      break;
+
+  if (i >= nw) {
+    meta_bug("Failed to find destination workspace in layout\n");
+    goto finish;
+  }
+
+  y = i / layout.cols;
+  x = i % layout.cols;
+
+  /* We priorize horizontal over vertical movements here. The
+     rationale for this is that horizontal movements are probably more
+     interesting for sound effects because speakers are usually
+     positioned on a horizontal and not a vertical axis. i.e. your
+     spatial "Woosh!" effects will easily be able to encode horizontal
+     movement but not such much vertical movement. */
+
+  if (x < layout.current_col)
+    e = "desktop-switch-left";
+  else if (x > layout.current_col)
+    e = "desktop-switch-right";
+  else if (y < layout.current_row)
+    e = "desktop-switch-up";
+  else if (y > layout.current_row)
+    e = "desktop-switch-down";
+  else {
+    meta_bug("Uh, origin and destination workspace at same logic position!\n");
+    goto finish;
+  }
+
+  ca_context_play(ca_gtk_context_get(), 1,
+                  CA_PROP_EVENT_ID, e,
+                  CA_PROP_EVENT_DESCRIPTION, "Desktop switched",
+                  NULL);
+
+ finish:
+  meta_screen_free_workspace_layout (&layout);
+}
+
 void
 meta_workspace_activate_with_focus (MetaWorkspace *workspace,
                                     MetaWindow    *focus_this,
@@ -467,6 +378,9 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
   if (workspace->screen->active_workspace == workspace)
     return;
 
+  if (workspace->screen->active_workspace)
+    workspace_switch_sound(workspace->screen->active_workspace, workspace);
+
   /* Note that old can be NULL; e.g. when starting up */
   old = workspace->screen->active_workspace;
   
@@ -530,60 +444,6 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
       meta_topic (META_DEBUG_FOCUS, "Focusing default window on new workspace\n");
       meta_workspace_focus_default_window (workspace, NULL, timestamp);
     }
-
-#ifdef HAVE_COMPOSITE_EXTENSIONS
-  {
-    /*
-     * Notify the compositor that the active workspace changed.
-     */
-    MetaScreen     *screen = workspace->screen;
-    MetaDisplay    *display = meta_screen_get_display (screen);
-    MetaCompositor *comp = meta_display_get_compositor (display);
-    MetaWorkspaceLayout layout1, layout2;
-    gint num_workspaces, current_space, new_space;
-    MetaMotionDirection direction = 0;
-
-    current_space = meta_workspace_index (old);
-    new_space     = meta_workspace_index (workspace);
-
-    num_workspaces = meta_screen_get_n_workspaces (workspace->screen);
-    meta_screen_calc_workspace_layout (workspace->screen, num_workspaces,
-                                       current_space, &layout1);
-
-    meta_screen_calc_workspace_layout (workspace->screen, num_workspaces,
-                                       new_space, &layout2);
-
-    if (layout1.current_col < layout2.current_col)
-      direction = META_MOTION_RIGHT;
-    if (layout1.current_col > layout2.current_col)
-      direction = META_MOTION_LEFT;
-
-    if (layout1.current_row < layout2.current_row)
-      {
-        if (!direction)
-          direction = META_MOTION_DOWN;
-        else if (direction == META_MOTION_RIGHT)
-          direction = META_MOTION_DOWN_RIGHT;
-        else
-          direction = META_MOTION_DOWN_LEFT;
-      }
-
-    if (layout1.current_row > layout2.current_row)
-      {
-        if (!direction)
-          direction = META_MOTION_UP;
-        else if (direction == META_MOTION_RIGHT)
-          direction = META_MOTION_UP_RIGHT;
-        else
-          direction = META_MOTION_UP_LEFT;
-      }
-
-    meta_screen_free_workspace_layout (&layout1);
-    meta_screen_free_workspace_layout (&layout2);
-
-    meta_compositor_switch_workspace (comp, screen, old, workspace, direction);
-  }
-#endif
 }
 
 void
@@ -606,20 +466,6 @@ meta_workspace_index (MetaWorkspace *workspace)
   return ret;
 }
 
-void
-meta_workspace_update_window_hints (MetaWorkspace *workspace)
-{
-  GList *l = workspace->windows;
-  while (l)
-    {
-      MetaWindow *win = l->data;
-
-      meta_window_set_current_workspace_hint (win);
-
-      l = l->next;
-    }
-}
-
 /* get windows contained on workspace, including workspace->windows
  * and also sticky windows.
  */
@@ -698,7 +544,7 @@ meta_workspace_invalidate_work_area (MetaWorkspace *workspace)
   g_free (workspace->work_area_xinerama);
   workspace->work_area_xinerama = NULL;
       
-  workspace_free_all_struts (workspace);
+  workspace_free_struts (workspace);
 
   for (i = 0; i < workspace->screen->n_xinerama_infos; i++)
     meta_rectangle_free_list_and_elements (workspace->xinerama_region[i]);
@@ -730,26 +576,6 @@ meta_workspace_invalidate_work_area (MetaWorkspace *workspace)
   meta_screen_queue_workarea_recalc (workspace->screen);
 }
 
-static MetaStrut *
-copy_strut(MetaStrut *original)
-{
-  return g_memdup(original, sizeof(MetaStrut));
-}
-
-static GSList *
-copy_strut_list(GSList *original)
-{
-  GSList *result = NULL;
-
-  while (original)
-    {
-      result = g_slist_prepend (result, copy_strut (original->data));
-      original = original->next;
-    }
-
-  return g_slist_reverse (result);
-}
-
 static void
 ensure_work_areas_validated (MetaWorkspace *workspace)
 {
@@ -767,10 +593,7 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
   g_assert (workspace->screen_edges == NULL);
   g_assert (workspace->xinerama_edges == NULL);
 
-  /* STEP 1: Get the list of struts */
-
-  workspace->all_struts = copy_strut_list (workspace->builtin_struts);
-
+  /* STEP 1: Get the list of struts */  
   windows = meta_workspace_list_windows (workspace);
   for (tmp = windows; tmp != NULL; tmp = tmp->next)
     {
@@ -778,8 +601,10 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
       GSList *s_iter;
 
       for (s_iter = win->struts; s_iter != NULL; s_iter = s_iter->next) {
+        MetaStrut *cpy = g_new (MetaStrut, 1);
+        *cpy = *((MetaStrut *)s_iter->data);
         workspace->all_struts = g_slist_prepend (workspace->all_struts,
-                                                 copy_strut(s_iter->data));
+                                                 cpy);
       }
     }
   g_list_free (windows);
@@ -918,38 +743,6 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
 
   /* We're all done, YAAY!  Record that everything has been validated. */
   workspace->work_areas_invalid = FALSE;
-
-#ifdef HAVE_COMPOSITE_EXTENSIONS
-  {
-    /*
-     * Notify the compositor that the workspace geometry has changed.
-     */
-    MetaScreen     *screen = workspace->screen;
-    MetaDisplay    *display = meta_screen_get_display (screen);
-    MetaCompositor *comp = meta_display_get_compositor (display);
-
-    meta_compositor_update_workspace_geometry (comp, workspace);
-  }
-#endif
-}
-
-/**
- * meta_workspace_set_builtin_struts:
- * @workspace: a #MetaWorkspace
- * @struts: (element-type Strut) (transfer none): list of #MetaStrut
- *
- * Sets a list of struts that will be used in addition to the struts
- * of the windows in the workspace when computing the work area of
- * the workspace.
- */
-void
-meta_workspace_set_builtin_struts (MetaWorkspace *workspace,
-                                   GSList        *struts)
-{
-  workspace_free_builtin_struts (workspace);
-  workspace->builtin_struts = copy_strut_list (struts);
-
-  meta_workspace_invalidate_work_area (workspace);
 }
 
 void
@@ -1005,14 +798,6 @@ meta_motion_direction_to_string (MetaMotionDirection direction)
       return "Left";
     case META_MOTION_RIGHT:
       return "Right";
-    case META_MOTION_UP_RIGHT:
-      return "Up-Right";
-    case META_MOTION_DOWN_RIGHT:
-      return "Down-Right";
-    case META_MOTION_UP_LEFT:
-      return "Up-Left";
-    case META_MOTION_DOWN_LEFT:
-      return "Down-Left";
     }
 
   return "Unknown";
@@ -1051,7 +836,6 @@ meta_workspace_get_neighbor (MetaWorkspace      *workspace,
     case META_MOTION_DOWN:
       layout.current_row += 1;
       break;
-    default:;
     }
 
   if (layout.current_col < 0)
@@ -1251,18 +1035,3 @@ focus_ancestor_or_mru_window (MetaWorkspace *workspace,
                                               timestamp);
     }
 }
-
-/**
- * meta_workspace_get_screen:
- * @workspace: a #MetaWorkspace
- *
- * Gets the #MetaScreen that the workspace is part of.
- *
- * Return value: (transfer none): the #MetaScreen for the workspace
- */
-MetaScreen *
-meta_workspace_get_screen (MetaWorkspace *workspace)
-{
-  return workspace->screen;
-}
-
