@@ -854,7 +854,8 @@ do_appwindow (void)
 
   /* Set up item factory to go away with the window */
   g_object_ref (item_factory);
-  gtk_object_sink (GTK_OBJECT (item_factory));
+  g_object_ref_sink (item_factory);
+  g_object_unref (item_factory);
   g_object_set_data_full (G_OBJECT (window),
                           "<main>",
                           item_factory,
